@@ -255,13 +255,13 @@ class AutogenContext(object):
         self.metadata = metadata = opts.get('target_metadata', None) \
             if metadata is None else metadata
 
-        if metadata is None and \
+        if autogenerate and metadata is None and \
                 migration_context is not None and \
                 migration_context.script is not None:
             raise util.CommandError(
                 "Can't proceed with --autogenerate option; environment "
                 "script %s does not provide "
-                "a MetaData object to the context." % (
+                "a MetaData object or sequence of objects to the context." % (
                     migration_context.script.env_py_location
                 ))
 
@@ -320,15 +320,61 @@ class AutogenContext(object):
         else:
             return True
 
+    @util.memoized_property
+    def sorted_tables(self):
+        """Return an aggregate of the :attr:`.MetaData.sorted_tables` collection(s).
+
+        For a sequence of :class:`.MetaData` objects, this
+        concatenates the :attr:`.MetaData.sorted_tables` collection
+        for each individual :class:`.MetaData`  in the order of the
+        sequence.  It does **not** collate the sorted tables collections.
+
+        .. versionadded:: 0.9.0
+
+        """
+        result = []
+        for m in util.to_list(self.metadata):
+            result.extend(m.sorted_tables)
+        return result
+
+    @util.memoized_property
+    def table_key_to_table(self):
+        """Return an aggregate  of the :attr:`.MetaData.tables` dictionaries.
+
+        The :attr:`.MetaData.tables` collection is a dictionary of table key
+        to :class:`.Table`; this method aggregates the dictionary across
+        multiple :class:`.MetaData` objects into one dictionary.
+
+        Duplicate table keys are **not** supported; if two :class:`.MetaData`
+        objects contain the same table key, an exception is raised.
+
+        .. versionadded:: 0.9.0
+
+        """
+        result = {}
+        for m in util.to_list(self.metadata):
+            intersect = set(result).intersection(set(m.tables))
+            if intersect:
+                raise ValueError(
+                    "Duplicate table keys across multiple "
+                    "MetaData objects: %s" %
+                    (", ".join('"%s"' % key for key in sorted(intersect)))
+                )
+
+            result.update(m.tables)
+        return result
+
 
 class RevisionContext(object):
     """Maintains configuration and state that's specific to a revision
     file generation operation."""
 
-    def __init__(self, config, script_directory, command_args):
+    def __init__(self, config, script_directory, command_args,
+                 process_revision_directives=None):
         self.config = config
         self.script_directory = script_directory
         self.command_args = command_args
+        self.process_revision_directives = process_revision_directives
         self.template_args = {
             'config': config  # Let templates use config for
                               # e.g. multiple databases
@@ -349,7 +395,7 @@ class RevisionContext(object):
             # renders
             autogen_context.imports = set()
             if migration_script.imports:
-                autogen_context.imports.union_update(migration_script.imports)
+                autogen_context.imports.update(migration_script.imports)
             render._render_python_into_templatevars(
                 autogen_context, migration_script, template_args
             )
@@ -404,6 +450,10 @@ class RevisionContext(object):
             compare._populate_migration_script(
                 autogen_context, migration_script)
 
+        if self.process_revision_directives:
+            self.process_revision_directives(
+                migration_context, rev, self.generated_revisions)
+
         hook = migration_context.opts['process_revision_directives']
         if hook:
             hook(migration_context, rev, self.generated_revisions)
