@@ -153,11 +153,27 @@ class Config(object):
         return {}
 
     def print_stdout(self, text, *arg):
-        """Render a message to standard out."""
+        """Render a message to standard out.
+
+        When :meth:`.Config.print_stdout` is called with additional args
+        those arguments will formatted against the provided text,
+        otherwise we simply output the provided text verbatim.
+
+        e.g.::
+
+            >>> config.print_stdout('Some text %s', 'arg')
+            Some Text arg
+
+        """
+
+        if arg:
+            output = (compat.text_type(text) % arg)
+        else:
+            output = compat.text_type(text)
 
         util.write_outstream(
             self.stdout,
-            (compat.text_type(text) % arg),
+            output,
             "\n"
         )
 
@@ -388,6 +404,13 @@ class CommandLine(object):
                         action="store",
                         help="Specify a revision range; "
                         "format is [start]:[end]")
+                ),
+                'indicate_current': (
+                    "-i", "--indicate-current",
+                    dict(
+                        action="store_true",
+                        help="Indicate the current revision"
+                    )
                 )
             }
             positional_help = {
@@ -432,7 +455,7 @@ class CommandLine(object):
                     fn.__name__[0] != '_' and \
                     fn.__module__ == 'alembic.command':
 
-                spec = inspect.getargspec(fn)
+                spec = compat.inspect_getargspec(fn)
                 if spec[3]:
                     positional = spec[0][1:-len(spec[3])]
                     kwarg = spec[0][-len(spec[3]):]
@@ -452,8 +475,8 @@ class CommandLine(object):
 
         try:
             fn(config,
-               *[getattr(options, k) for k in positional],
-               **dict((k, getattr(options, k)) for k in kwarg)
+               *[getattr(options, k, None) for k in positional],
+               **dict((k, getattr(options, k, None)) for k in kwarg)
                )
         except util.CommandError as e:
             if options.raiseerr:
