@@ -6,6 +6,8 @@ from .base import alter_table, AddColumn, ColumnName, RenameTable,\
     format_table_name, format_column_name, ColumnNullable, alter_column,\
     format_server_default, ColumnDefault, format_type, ColumnType
 from sqlalchemy.sql.expression import ClauseElement, Executable
+from sqlalchemy.schema import CreateIndex, Column
+from sqlalchemy import types as sqltypes
 
 
 class MSSQLImpl(DefaultImpl):
@@ -39,11 +41,10 @@ class MSSQLImpl(DefaultImpl):
                      name=None,
                      type_=None,
                      schema=None,
-                     autoincrement=None,
                      existing_type=None,
                      existing_server_default=None,
                      existing_nullable=None,
-                     existing_autoincrement=None
+                     **kw
                      ):
 
         if nullable is not None and existing_type is None:
@@ -63,10 +64,9 @@ class MSSQLImpl(DefaultImpl):
             nullable=nullable,
             type_=type_,
             schema=schema,
-            autoincrement=autoincrement,
             existing_type=existing_type,
             existing_nullable=existing_nullable,
-            existing_autoincrement=existing_autoincrement
+            **kw
         )
 
         if server_default is not False:
@@ -89,6 +89,16 @@ class MSSQLImpl(DefaultImpl):
                 schema=schema,
                 name=name)
 
+    def create_index(self, index):
+        # this likely defaults to None if not present, so get()
+        # should normally not return the default value.  being
+        # defensive in any case
+        mssql_include = index.kwargs.get('mssql_include', None) or ()
+        for col in mssql_include:
+            if col not in index.table.c:
+                index.table.append_column(Column(col, sqltypes.NullType))
+        self._exec(CreateIndex(index))
+
     def bulk_insert(self, table, rows, **kw):
         if self.as_sql:
             self._exec(
@@ -231,5 +241,6 @@ def visit_column_type(element, compiler, **kw):
 def visit_rename_table(element, compiler, **kw):
     return "EXEC sp_rename '%s', %s" % (
         format_table_name(compiler, element.table_name, element.schema),
-        format_table_name(compiler, element.new_table_name, element.schema)
+        format_table_name(compiler, element.new_table_name, None)
     )
+
