@@ -5,13 +5,12 @@ import re
 from sqlalchemy import create_engine, text, MetaData
 
 import alembic
-from alembic.compat import configparser
-from alembic import util
-from alembic.compat import string_types, text_type
-from alembic.migration import MigrationContext
-from alembic.environment import EnvironmentContext
-from alembic.operations import Operations
-from alembic.ddl.impl import _impls
+from ..util.compat import configparser
+from .. import util
+from ..util.compat import string_types, text_type
+from ..migration import MigrationContext
+from ..environment import EnvironmentContext
+from ..operations import Operations
 from contextlib import contextmanager
 from .plugin.plugin_base import SkipTest
 from .assertions import _get_dialect, eq_
@@ -90,7 +89,8 @@ def capture_context_buffer(**kw):
 
 def op_fixture(
         dialect='default', as_sql=False,
-        naming_convention=None, literal_binds=False):
+        naming_convention=None, literal_binds=False,
+        native_boolean=None):
 
     opts = {}
     if naming_convention:
@@ -111,7 +111,7 @@ def op_fixture(
                 # the impl produces soft tabs,
                 # so search for blocks of 4 spaces
                 msg = re.sub(r'    ', '', msg)
-                msg = re.sub('\;\n*$', '', msg)
+                msg = re.sub(r'\;\n*$', '', msg)
 
             self.lines.append(msg)
 
@@ -144,6 +144,11 @@ def op_fixture(
     if literal_binds:
         opts['literal_binds'] = literal_binds
     ctx_dialect = _get_dialect(dialect)
+    if native_boolean is not None:
+        ctx_dialect.supports_native_boolean = native_boolean
+        # this is new as of SQLAlchemy 1.2.7 and is used by SQL Server,
+        # which breaks assumptions in the alembic test suite
+        ctx_dialect.non_native_boolean_check_constraint = True
     if not as_sql:
         def execute(stmt, *multiparam, **param):
             if isinstance(stmt, string_types):
