@@ -25,6 +25,7 @@ py3k = sys.version_info >= (3, )
 
 class AutogenCrossSchemaTest(AutogenTest, TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
 
     @classmethod
     def _get_db_schema(cls):
@@ -139,6 +140,7 @@ class AutogenCrossSchemaTest(AutogenTest, TestBase):
 
 class AutogenDefaultSchemaTest(AutogenFixtureTest, TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
 
     def test_uses_explcit_schema_in_default_one(self):
 
@@ -512,12 +514,16 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         )
         eq_(
             [(rec[0], rec[1].name) for rec in uo.as_diffs()],
-            [('remove_table', 'extra'), ('remove_table', 'user')]
+            [
+                ('remove_table', 'extra'),
+                ('remove_index', 'pw_idx'),
+                ('remove_table', 'user'), ]
         )
 
 
 class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
     schema = "test_schema"
 
     def test_diffs(self):
@@ -635,6 +641,13 @@ class CompareTypeSpecificityTest(TestBase):
         is_(impl.compare_type(Column('x', t3), Column('x', t2)), True)
         is_(impl.compare_type(Column('x', t3), Column('x', t4)), True)
 
+    def test_numeric_noprecision(self):
+        t1 = Numeric()
+        t2 = Numeric(scale=5)
+
+        impl = self._fixture()
+        is_(impl.compare_type(Column('x', t1), Column('x', t2)), False)
+
     def test_integer(self):
         t1 = Integer()
         t2 = SmallInteger()
@@ -662,6 +675,80 @@ class CompareTypeSpecificityTest(TestBase):
         is_(impl.compare_type(Column('x', t2), Column('x', t3)), True)
 
 
+class AutogenSystemColTest(AutogenTest, TestBase):
+    __only_on__ = 'postgresql'
+
+    @classmethod
+    def _get_db_schema(cls):
+        m = MetaData()
+
+        Table(
+            'sometable', m,
+            Column('id', Integer, primary_key=True),
+        )
+        return m
+
+    @classmethod
+    def _get_model_schema(cls):
+        m = MetaData()
+
+        # 'xmin' is implicitly present, when added to a model should produce
+        # no change
+        Table(
+            'sometable', m,
+            Column('id', Integer, primary_key=True),
+            Column('xmin', Integer, system=True)
+        )
+        return m
+
+    def test_dont_add_system(self):
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(self.autogen_context, uo)
+
+        diffs = uo.as_diffs()
+        eq_(diffs, [])
+
+
+class AutogenerateVariantCompareTest(AutogenTest, TestBase):
+    __backend__ = True
+
+    # 1.0.13 and lower fail on Postgresql due to variant / bigserial issue
+    # #3739
+
+    __requires__ = ('sqlalchemy_1014', )
+
+    @classmethod
+    def _get_db_schema(cls):
+        m = MetaData()
+
+        Table('sometable', m,
+              Column(
+                  'id',
+                  BigInteger().with_variant(Integer, "sqlite"),
+                  primary_key=True),
+              Column('value', String(50)))
+        return m
+
+    @classmethod
+    def _get_model_schema(cls):
+        m = MetaData()
+
+        Table('sometable', m,
+              Column(
+                  'id',
+                  BigInteger().with_variant(Integer, "sqlite"),
+                  primary_key=True),
+              Column('value', String(50)))
+        return m
+
+    def test_variant_no_issue(self):
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(self.autogen_context, uo)
+
+        diffs = uo.as_diffs()
+        eq_(diffs, [])
+
+
 class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
     __only_on__ = 'sqlite'
 
@@ -842,6 +929,7 @@ class AutogenVersionTableTest(AutogenTest, TestBase):
 
 class AutogenCustomVersionTableSchemaTest(AutogenVersionTableTest):
     __only_on__ = 'postgresql'
+    __backend__ = True
     version_table_schema = 'test_schema'
     configure_opts = {'version_table_schema': 'test_schema'}
 
@@ -853,6 +941,7 @@ class AutogenCustomVersionTableTest(AutogenVersionTableTest):
 
 class AutogenCustomVersionTableAndSchemaTest(AutogenVersionTableTest):
     __only_on__ = 'postgresql'
+    __backend__ = True
     version_table_name = 'my_version_table'
     version_table_schema = 'test_schema'
     configure_opts = {
@@ -1039,6 +1128,7 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestBase):
 
 class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
     schema = "test_schema"
 
     def test_compare_metadata_schema(self):
@@ -1078,7 +1168,6 @@ class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
         eq_(diffs[5][0][5], False)
         eq_(diffs[5][0][6], True)
 
-
 class OrigObjectTest(TestBase):
     def setUp(self):
         self.metadata = m = MetaData()
