@@ -1,17 +1,18 @@
 import re
 import sys
 from alembic.testing import TestBase, exclusions, assert_raises
+from alembic.testing import assertions
 
 from alembic.operations import ops
 from sqlalchemy import MetaData, Column, Table, String, \
-    Numeric, CHAR, ForeignKey, DATETIME, Integer, \
+    Numeric, CHAR, ForeignKey, DATETIME, Integer, BigInteger, \
     CheckConstraint, Unicode, Enum, cast,\
-    UniqueConstraint, Boolean, ForeignKeyConstraint,\
+    DateTime, UniqueConstraint, Boolean, ForeignKeyConstraint,\
     PrimaryKeyConstraint, Index, func, text, DefaultClause
 
 from sqlalchemy.types import TIMESTAMP
+from sqlalchemy import types
 from sqlalchemy.types import UserDefinedType
-from sqlalchemy.dialects import mysql, postgresql
 from sqlalchemy.engine.default import DefaultDialect
 from sqlalchemy.sql import and_, column, literal_column, false, table
 from alembic.migration import MigrationContext
@@ -41,24 +42,12 @@ class AutogenRenderTest(TestBase):
             'target_metadata': MetaData()
         }
         context = MigrationContext.configure(
-            dialect_name="mysql",
+            dialect=DefaultDialect(),
             opts=ctx_opts
         )
 
         self.autogen_context = api.AutogenContext(context)
 
-        context = MigrationContext.configure(
-            dialect_name="postgresql",
-            opts=ctx_opts
-        )
-        self.pg_autogen_context = api.AutogenContext(context)
-
-        context = MigrationContext.configure(
-            dialect=DefaultDialect(),
-            opts=ctx_opts
-        )
-        self.default_autogen_context = api.AutogenContext(context)
-
     def test_render_add_index(self):
         """
         autogenerate.render._add_index
@@ -135,36 +124,6 @@ class AutogenRenderTest(TestBase):
                 "['active', 'code'], unique=False)"
             )
 
-    def test_render_add_index_pg_where(self):
-        autogen_context = self.pg_autogen_context
-
-        m = MetaData()
-        t = Table('t', m,
-                  Column('x', String),
-                  Column('y', String)
-                  )
-
-        idx = Index('foo_idx', t.c.x, t.c.y,
-                    postgresql_where=(t.c.y == 'something'))
-
-        op_obj = ops.CreateIndexOp.from_index(idx)
-
-        if compat.sqla_08:
-            eq_ignore_whitespace(
-                autogenerate.render_op_text(autogen_context, op_obj),
-                """op.create_index('foo_idx', 't', \
-['x', 'y'], unique=False, """
-                """postgresql_where=sa.text(!U"y = 'something'"))"""
-            )
-        else:
-            eq_ignore_whitespace(
-                autogenerate.render_op_text(autogen_context, op_obj),
-                """op.create_index('foo_idx', 't', ['x', 'y'], \
-unique=False, """
-                """postgresql_where=sa.text(!U't.y = %(y_1)s'))"""
-            )
-
-    @config.requirements.fail_before_sqla_080
     def test_render_add_index_func(self):
         m = MetaData()
         t = Table(
@@ -181,7 +140,6 @@ unique=False, """
             "[sa.text(!U'lower(code)')], unique=False)"
         )
 
-    @config.requirements.fail_before_sqla_080
     def test_render_add_index_cast(self):
         m = MetaData()
         t = Table(
@@ -196,16 +154,15 @@ unique=False, """
             eq_ignore_whitespace(
                 autogenerate.render_op_text(self.autogen_context, op_obj),
                 "op.create_index('test_lower_code_idx', 'test', "
-                "[sa.text(!U'CAST(code AS CHAR)')], unique=False)"
+                "[sa.text(!U'CAST(code AS VARCHAR)')], unique=False)"
             )
         else:
             eq_ignore_whitespace(
                 autogenerate.render_op_text(self.autogen_context, op_obj),
                 "op.create_index('test_lower_code_idx', 'test', "
-                "[sa.text(!U'CAST(test.code AS CHAR)')], unique=False)"
+                "[sa.text(!U'CAST(code AS VARCHAR)')], unique=False)"
             )
 
-    @config.requirements.fail_before_sqla_080
     def test_render_add_index_desc(self):
         m = MetaData()
         t = Table(
@@ -404,6 +361,24 @@ unique=False, """
             "schema='CamelSchema', type_='unique')"
         )
 
+    def test_drop_unique_constraint_schema_reprobj(self):
+        """
+        autogenerate.render._drop_constraint using schema
+        """
+        class SomeObj(str):
+            def __repr__(self):
+                return "foo.camel_schema"
+
+        op_obj = ops.DropConstraintOp(
+            "uq_test_code", "test", type_="unique",
+            schema=SomeObj("CamelSchema")
+        )
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.drop_constraint('uq_test_code', 'test', "
+            "schema=foo.camel_schema, type_='unique')"
+        )
+
     def test_add_fk_constraint(self):
         m = MetaData()
         Table('a', m, Column('id', Integer, primary_key=True))
@@ -435,8 +410,6 @@ unique=False, """
         t2 = Table('t2', m, Column('c_rem', Integer))
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], onupdate="CASCADE")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
 
         # SQLA 0.9 generates a u'' here for remote cols while 0.8 does not,
         # so just whack out "'u" here from the generated
@@ -452,8 +425,6 @@ unique=False, """
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], ondelete="CASCADE")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
 
         op_obj = ops.AddConstraintOp.from_constraint(fk)
         eq_ignore_whitespace(
@@ -466,8 +437,6 @@ unique=False, """
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], deferrable=True)
-        if not util.sqla_08:
-            t1.append_constraint(fk)
         op_obj = ops.AddConstraintOp.from_constraint(fk)
         eq_ignore_whitespace(
             re.sub(
@@ -479,8 +448,6 @@ unique=False, """
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], initially="XYZ")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
         op_obj = ops.AddConstraintOp.from_constraint(fk)
         eq_ignore_whitespace(
             re.sub(
@@ -494,8 +461,6 @@ unique=False, """
         fk = ForeignKeyConstraint(
             [t1.c.c], [t2.c.c_rem],
             initially="XYZ", ondelete="CASCADE", deferrable=True)
-        if not util.sqla_08:
-            t1.append_constraint(fk)
         op_obj = ops.AddConstraintOp.from_constraint(fk)
         eq_ignore_whitespace(
             re.sub(
@@ -723,6 +688,21 @@ unique=False, """
             ")"
         )
 
+    def test_render_table_w_system(self):
+        m = MetaData()
+        t = Table('sometable', m,
+                  Column('id', Integer, primary_key=True),
+                  Column('xmin', Integer, system=True, nullable=False)
+                  )
+        op_obj = ops.CreateTableOp.from_table(t)
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.create_table('sometable',"
+            "sa.Column('id', sa.Integer(), nullable=False),"
+            "sa.Column('xmin', sa.Integer(), nullable=False, system=True),"
+            "sa.PrimaryKeyConstraint('id'))"
+        )
+
     def test_render_table_w_unicode_name(self):
         m = MetaData()
         t = Table(compat.ue('\u0411\u0435\u0437'), m,
@@ -751,6 +731,29 @@ unique=False, """
             "schema=%r)" % compat.ue('\u0411\u0435\u0437')
         )
 
+    def test_render_table_w_unsupported_constraint(self):
+        from sqlalchemy.sql.schema import ColumnCollectionConstraint
+
+        class SomeCustomConstraint(ColumnCollectionConstraint):
+            __visit_name__ = 'some_custom'
+
+        m = MetaData()
+
+        t = Table(
+            't', m, Column('id', Integer),
+            SomeCustomConstraint('id'),
+        )
+        op_obj = ops.CreateTableOp.from_table(t)
+        with assertions.expect_warnings(
+                "No renderer is established for object SomeCustomConstraint"):
+            eq_ignore_whitespace(
+                autogenerate.render_op_text(self.autogen_context, op_obj),
+                "op.create_table('t',"
+                "sa.Column('id', sa.Integer(), nullable=True),"
+                "[Unknown Python object "
+                "SomeCustomConstraint(Column('id', Integer(), table=<t>))])"
+            )
+
     @patch("alembic.autogenerate.render.MAX_PYTHON_ARGS", 3)
     def test_render_table_max_cols(self):
         m = MetaData()
@@ -922,6 +925,24 @@ unique=False, """
             "sa.PrimaryKeyConstraint('x'))"
         )
 
+    @config.requirements.fail_before_sqla_110
+    def test_render_table_w_autoincrement(self):
+        m = MetaData()
+        t = Table(
+            'test', m,
+            Column('id1', Integer, primary_key=True),
+            Column('id2', Integer, primary_key=True, autoincrement=True))
+        op_obj = ops.CreateTableOp.from_table(t)
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.create_table('test',"
+            "sa.Column('id1', sa.Integer(), nullable=False),"
+            "sa.Column('id2', sa.Integer(), autoincrement=True, "
+            "nullable=False),"
+            "sa.PrimaryKeyConstraint('id1', 'id2')"
+            ")"
+        )
+
     def test_render_add_column(self):
         op_obj = ops.AddColumnOp(
             "foo", Column("x", Integer, server_default="5"))
@@ -931,6 +952,18 @@ unique=False, """
             "server_default='5', nullable=True))"
         )
 
+    def test_render_add_column_system(self):
+        # this would never actually happen since "system" columns
+        # can't be added in any case.   Howver it will render as
+        # part of op.CreateTableOp.
+        op_obj = ops.AddColumnOp(
+            "foo", Column("xmin", Integer, system=True))
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.add_column('foo', sa.Column('xmin', sa.Integer(), "
+            "nullable=True, system=True))"
+        )
+
     def test_render_add_column_w_schema(self):
         op_obj = ops.AddColumnOp(
             "bar", Column("x", Integer, server_default="5"),
@@ -1116,14 +1149,25 @@ unique=False, """
             "existing_type=sa.Integer(), nullable=True, schema='foo')"
         )
 
+    def test_render_modify_type_w_autoincrement(self):
+        op_obj = ops.AlterColumnOp(
+            "sometable", "somecolumn",
+            modify_type=Integer(), existing_type=BigInteger(),
+            autoincrement=True
+        )
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.alter_column('sometable', 'somecolumn', "
+            "existing_type=sa.BigInteger(), type_=sa.Integer(), "
+            "autoincrement=True)"
+        )
+
     def test_render_fk_constraint_kwarg(self):
         m = MetaData()
         t1 = Table('t', m, Column('c', Integer))
         t2 = Table('t2', m, Column('c_rem', Integer))
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], onupdate="CASCADE")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
 
         # SQLA 0.9 generates a u'' here for remote cols while 0.8 does not,
         # so just whack out "'u" here from the generated
@@ -1137,8 +1181,6 @@ unique=False, """
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], ondelete="CASCADE")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
 
         eq_ignore_whitespace(
             re.sub(
@@ -1149,8 +1191,6 @@ unique=False, """
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], deferrable=True)
-        if not util.sqla_08:
-            t1.append_constraint(fk)
         eq_ignore_whitespace(
             re.sub(
                 r"u'", "'",
@@ -1161,8 +1201,6 @@ unique=False, """
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], initially="XYZ")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
         eq_ignore_whitespace(
             re.sub(
                 r"u'", "'",
@@ -1175,8 +1213,6 @@ unique=False, """
         fk = ForeignKeyConstraint(
             [t1.c.c], [t2.c.c_rem],
             initially="XYZ", ondelete="CASCADE", deferrable=True)
-        if not util.sqla_08:
-            t1.append_constraint(fk)
         eq_ignore_whitespace(
             re.sub(
                 r"u'", "'",
@@ -1187,6 +1223,75 @@ unique=False, """
             "ondelete='CASCADE', initially='XYZ', deferrable=True)"
         )
 
+    def test_render_fk_constraint_resolve_key(self):
+        m = MetaData()
+        t1 = Table('t', m, Column('c', Integer))
+        t2 = Table('t2', m, Column('c_rem', Integer, key='c_remkey'))
+
+        fk = ForeignKeyConstraint(['c'], ['t2.c_remkey'])
+        t1.append_constraint(fk)
+
+        eq_ignore_whitespace(
+            re.sub(
+                r"u'", "'",
+                autogenerate.render._render_constraint(
+                    fk, self.autogen_context)),
+            "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], )"
+        )
+
+    def test_render_fk_constraint_bad_table_resolve(self):
+        m = MetaData()
+        t1 = Table('t', m, Column('c', Integer))
+        t2 = Table('t2', m, Column('c_rem', Integer))
+
+        fk = ForeignKeyConstraint(['c'], ['t2.nonexistent'])
+        t1.append_constraint(fk)
+
+        eq_ignore_whitespace(
+            re.sub(
+                r"u'", "'",
+                autogenerate.render._render_constraint(
+                    fk, self.autogen_context)),
+            "sa.ForeignKeyConstraint(['c'], ['t2.nonexistent'], )"
+        )
+
+    def test_render_fk_constraint_bad_table_resolve_dont_get_confused(self):
+        m = MetaData()
+        t1 = Table('t', m, Column('c', Integer))
+        t2 = Table(
+            't2', m,
+            Column('c_rem', Integer, key='cr_key'),
+            Column('c_rem_2', Integer, key='c_rem')
+
+        )
+
+        fk = ForeignKeyConstraint(['c'], ['t2.c_rem'], link_to_name=True)
+        t1.append_constraint(fk)
+
+        eq_ignore_whitespace(
+            re.sub(
+                r"u'", "'",
+                autogenerate.render._render_constraint(
+                    fk, self.autogen_context)),
+            "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], )"
+        )
+
+    def test_render_fk_constraint_link_to_name(self):
+        m = MetaData()
+        t1 = Table('t', m, Column('c', Integer))
+        t2 = Table('t2', m, Column('c_rem', Integer, key='c_remkey'))
+
+        fk = ForeignKeyConstraint(['c'], ['t2.c_rem'], link_to_name=True)
+        t1.append_constraint(fk)
+
+        eq_ignore_whitespace(
+            re.sub(
+                r"u'", "'",
+                autogenerate.render._render_constraint(
+                    fk, self.autogen_context)),
+            "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], )"
+        )
+
     def test_render_fk_constraint_use_alter(self):
         m = MetaData()
         Table('t', m, Column('c', Integer))
@@ -1210,8 +1315,6 @@ unique=False, """
         t2 = Table('t2', m, Column('c_rem', Integer))
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], onupdate="CASCADE")
-        if not util.sqla_08:
-            t1.append_constraint(fk)
 
         eq_ignore_whitespace(
             re.sub(
@@ -1244,7 +1347,6 @@ unique=False, """
             "sa.CheckConstraint(!U'c > 5 AND c < 10')"
         )
 
-    @config.requirements.fail_before_sqla_080
     def test_render_check_constraint_literal_binds(self):
         c = column('c')
         eq_ignore_whitespace(
@@ -1293,7 +1395,6 @@ unique=False, """
             "existing_server_default='5')"
         )
 
-    @config.requirements.fail_before_sqla_079
     def test_render_enum(self):
         eq_ignore_whitespace(
             autogenerate.render._repr_type(
@@ -1308,10 +1409,7 @@ unique=False, """
             "sa.Enum('one', 'two', 'three')"
         )
 
-    @exclusions.fails_if(
-        lambda config: (util.sqla_09 and not util.sqla_099) or not util.sqla_079,
-        "Fails on SQLAlchemy <0.7.9, 0.9.0-0.9.8"
-    )
+    @config.requirements.sqlalchemy_099
     def test_render_non_native_enum(self):
         eq_ignore_whitespace(
             autogenerate.render._repr_type(
@@ -1327,6 +1425,39 @@ unique=False, """
             "sa.Integer()"
         )
 
+    @config.requirements.sqlalchemy_110
+    def test_generic_array_type(self):
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                types.ARRAY(Integer), self.autogen_context),
+            "sa.ARRAY(sa.Integer())"
+        )
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                types.ARRAY(DateTime(timezone=True)), self.autogen_context),
+            "sa.ARRAY(sa.DateTime(timezone=True))"
+        )
+
+    @config.requirements.sqlalchemy_110
+    def test_render_array_no_context(self):
+        uo = ops.UpgradeOps(ops=[
+            ops.CreateTableOp(
+                "sometable",
+                [Column('x', types.ARRAY(Integer))]
+            )
+        ])
+
+        eq_(
+            autogenerate.render_python_code(uo),
+            "# ### commands auto generated by Alembic - please adjust! ###\n"
+            "    op.create_table('sometable',\n"
+            "    sa.Column('x', sa.ARRAY(sa.Integer()), nullable=True)\n"
+            "    )\n"
+            "    # ### end Alembic commands ###"
+        )
+
     def test_repr_custom_type_w_sqla_prefix(self):
         self.autogen_context.opts['user_module_prefix'] = None
 
@@ -1373,7 +1504,6 @@ unique=False, """
             "user.MyType()"
         )
 
-    @config.requirements.sqlalchemy_09
     def test_repr_dialect_type(self):
         from sqlalchemy.dialects.mysql import VARCHAR
 
@@ -1404,22 +1534,6 @@ unique=False, """
             'nullable=False)'
         )
 
-    def test_render_server_default_native_boolean(self):
-        c = Column(
-            'updated_at', Boolean(),
-            server_default=false(),
-            nullable=False)
-        result = autogenerate.render._render_column(
-            c, self.pg_autogen_context,
-        )
-        eq_ignore_whitespace(
-            result,
-            'sa.Column(\'updated_at\', sa.Boolean(), '
-            'server_default=sa.text(!U\'false\'), '
-            'nullable=False)'
-        )
-
-    @config.requirements.fail_before_sqla_09
     def test_render_server_default_non_native_boolean(self):
         c = Column(
             'updated_at', Boolean(),
@@ -1427,7 +1541,7 @@ unique=False, """
             nullable=False)
 
         result = autogenerate.render._render_column(
-            c, self.default_autogen_context
+            c, self.autogen_context
         )
         eq_ignore_whitespace(
             result,
