@@ -13,15 +13,14 @@ from alembic.runtime.migration import MigrationContext
 
 from sqlalchemy import Integer, Table, Column, String, MetaData, ForeignKey, \
     UniqueConstraint, ForeignKeyConstraint, Index, Boolean, CheckConstraint, \
-    Enum
+    Enum, DateTime, PrimaryKeyConstraint
 from sqlalchemy.engine.reflection import Inspector
-from sqlalchemy.sql import column, text
+from sqlalchemy.sql import column, text, select
 from sqlalchemy.schema import CreateTable, CreateIndex
 from sqlalchemy import exc
 
 
 class BatchApplyTest(TestBase):
-    __requires__ = ('sqlalchemy_08', )
 
     def setUp(self):
         self.op = Operations(mock.Mock(opts={}))
@@ -34,7 +33,7 @@ class BatchApplyTest(TestBase):
             Column('x', String(10)),
             Column('y', Integer)
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _uq_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -45,7 +44,7 @@ class BatchApplyTest(TestBase):
             Column('y', Integer),
             UniqueConstraint('y', name='uq1')
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _ix_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -56,7 +55,18 @@ class BatchApplyTest(TestBase):
             Column('y', Integer),
             Index('ix1', 'y')
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
+
+    def _pk_fixture(self):
+        m = MetaData()
+        t = Table(
+            'tname', m,
+            Column('id', Integer),
+            Column('x', String()),
+            Column('y', Integer),
+            PrimaryKeyConstraint('id', name="mypk")
+        )
+        return ApplyBatchImpl(t, (), {}, False)
 
     def _literal_ck_fixture(
             self, copy_from=None, table_args=(), table_kwargs={}):
@@ -70,7 +80,7 @@ class BatchApplyTest(TestBase):
                 Column('email', String()),
                 CheckConstraint("email LIKE '%@%'")
             )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _sql_ck_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -80,7 +90,7 @@ class BatchApplyTest(TestBase):
             Column('email', String())
         )
         t.append_constraint(CheckConstraint(t.c.email.like('%@%')))
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _fk_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -90,7 +100,7 @@ class BatchApplyTest(TestBase):
             Column('email', String()),
             Column('user_id', Integer, ForeignKey('user.id'))
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _multi_fk_fixture(self, table_args=(), table_kwargs={}, schema=None):
         m = MetaData()
@@ -112,7 +122,7 @@ class BatchApplyTest(TestBase):
                 ['%suser.id' % schemaarg, '%suser.id_version' % schemaarg]),
             schema=schema
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _named_fk_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -122,7 +132,7 @@ class BatchApplyTest(TestBase):
             Column('email', String()),
             Column('user_id', Integer, ForeignKey('user.id', name='ufk'))
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _selfref_fk_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -132,7 +142,7 @@ class BatchApplyTest(TestBase):
             Column('parent_id', Integer, ForeignKey('tname.id')),
             Column('data', String)
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _boolean_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -141,7 +151,7 @@ class BatchApplyTest(TestBase):
             Column('id', Integer, primary_key=True),
             Column('flag', Boolean)
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _boolean_no_ck_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -150,7 +160,7 @@ class BatchApplyTest(TestBase):
             Column('id', Integer, primary_key=True),
             Column('flag', Boolean(create_constraint=False))
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _enum_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -159,7 +169,7 @@ class BatchApplyTest(TestBase):
             Column('id', Integer, primary_key=True),
             Column('thing', Enum('a', 'b', 'c'))
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _server_default_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -168,7 +178,7 @@ class BatchApplyTest(TestBase):
             Column('id', Integer, primary_key=True),
             Column('thing', String(), server_default='')
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False)
 
     def _assert_impl(self, impl, colnames=None,
                      ddl_contains=None, ddl_not_contains=None,
@@ -194,7 +204,8 @@ class BatchApplyTest(TestBase):
         for idx in impl.new_indexes.values():
             impl.new_table.name = impl.table.name
             idx_stmt += str(CreateIndex(idx).compile(dialect=context.dialect))
-            impl.new_table.name = '_alembic_batch_temp'
+            impl.new_table.name = ApplyBatchImpl._calc_temp_name(
+                impl.table.name)
         idx_stmt = re.sub(r'[\n\t]', '', idx_stmt)
 
         if ddl_contains:
@@ -211,10 +222,13 @@ class BatchApplyTest(TestBase):
         else:
             args = {"schema": ""}
 
+        args["temp_name"] = impl.new_table.name
+
         args['colnames'] = ", ".join([
             impl.new_table.c[name].name
             for name in colnames
             if name in impl.table.c])
+
         args['tname_colnames'] = ", ".join(
             "CAST(%(schema)stname.%(name)s AS %(type)s) AS anon_1" % {
                 'schema': args['schema'],
@@ -230,10 +244,10 @@ class BatchApplyTest(TestBase):
         )
 
         expected.extend([
-            'INSERT INTO %(schema)s_alembic_batch_temp (%(colnames)s) '
+            'INSERT INTO %(schema)s%(temp_name)s (%(colnames)s) '
             'SELECT %(tname_colnames)s FROM %(schema)stname' % args,
             'DROP TABLE %(schema)stname' % args,
-            'ALTER TABLE %(schema)s_alembic_batch_temp '
+            'ALTER TABLE %(schema)s%(temp_name)s '
             'RENAME TO %(schema)stname' % args
         ])
         if idx_stmt:
@@ -243,9 +257,9 @@ class BatchApplyTest(TestBase):
 
     def test_change_type(self):
         impl = self._simple_fixture()
-        impl.alter_column('tname', 'x', type_=Integer)
+        impl.alter_column('tname', 'x', type_=String)
         new_table = self._assert_impl(impl)
-        assert new_table.c.x.type._type_affinity is Integer
+        assert new_table.c.x.type._type_affinity is String
 
     def test_rename_col(self):
         impl = self._simple_fixture()
@@ -405,9 +419,6 @@ class BatchApplyTest(TestBase):
             ddl_contains='FOREIGN KEY(user_id_3, user_id_version) '
             'REFERENCES "user" (id, id_version)')
 
-    # _get_colspec() in 0.8 calls upon fk.column when schema is
-    # present.  not sure if we want to try to fix this
-    @config.requirements.fail_before_sqla_09
     def test_regen_multi_fk_schema(self):
         impl = self._multi_fk_fixture(schema='foo_schema')
         self._assert_impl(
@@ -533,9 +544,16 @@ class BatchApplyTest(TestBase):
             dialect='mysql'
         )
 
+    def test_drop_pk(self):
+        impl = self._pk_fixture()
+        pk = self.op.schema_obj.primary_key_constraint("mypk", "tname", ["id"])
+        impl.drop_constraint(pk)
+        new_table = self._assert_impl(impl)
+        assert not new_table.c.id.primary_key
+        assert not len(new_table.primary_key)
+
 
 class BatchAPITest(TestBase):
-    __requires__ = ('sqlalchemy_08', )
 
     @contextmanager
     def _fixture(self, schema=None):
@@ -731,7 +749,6 @@ class BatchAPITest(TestBase):
 
 
 class CopyFromTest(TestBase):
-    __requires__ = ('sqlalchemy_08', )
 
     def _fixture(self):
         self.metadata = MetaData()
@@ -751,14 +768,13 @@ class CopyFromTest(TestBase):
         with self.op.batch_alter_table(
                 "foo", copy_from=self.table) as batch_op:
             batch_op.alter_column('data', type_=Integer)
-
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data INTEGER, x INTEGER, PRIMARY KEY (id))',
-            'INSERT INTO _alembic_batch_temp (id, data, x) SELECT foo.id, '
+            'INSERT INTO _alembic_tmp_foo (id, data, x) SELECT foo.id, '
             'CAST(foo.data AS INTEGER) AS anon_1, foo.x FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo'
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo'
         )
 
     def test_change_type_from_schematype(self):
@@ -774,12 +790,12 @@ class CopyFromTest(TestBase):
                 existing_type=Boolean(
                     create_constraint=True, name="ck1"))
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data VARCHAR(50), x INTEGER, y INTEGER, PRIMARY KEY (id))',
-            'INSERT INTO _alembic_batch_temp (id, data, x, y) SELECT foo.id, '
+            'INSERT INTO _alembic_tmp_foo (id, data, x, y) SELECT foo.id, '
             'foo.data, foo.x, CAST(foo.y AS INTEGER) AS anon_1 FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo'
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo'
         )
 
     def test_change_type_to_schematype(self):
@@ -794,13 +810,13 @@ class CopyFromTest(TestBase):
                 type_=Boolean(
                     create_constraint=True, name="ck1"))
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data VARCHAR(50), x INTEGER, y BOOLEAN, PRIMARY KEY (id), '
             'CONSTRAINT ck1 CHECK (y IN (0, 1)))',
-            'INSERT INTO _alembic_batch_temp (id, data, x, y) SELECT foo.id, '
+            'INSERT INTO _alembic_tmp_foo (id, data, x, y) SELECT foo.id, '
             'foo.data, foo.x, CAST(foo.y AS BOOLEAN) AS anon_1 FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo'
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo'
         )
 
     def test_create_drop_index_w_always(self):
@@ -811,13 +827,13 @@ class CopyFromTest(TestBase):
                 'ix_data', ['data'], unique=True)
 
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data VARCHAR(50), '
             'x INTEGER, PRIMARY KEY (id))',
-            'INSERT INTO _alembic_batch_temp (id, data, x) '
+            'INSERT INTO _alembic_tmp_foo (id, data, x) '
             'SELECT foo.id, foo.data, foo.x FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo',
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo',
             'CREATE UNIQUE INDEX ix_data ON foo (data)',
         )
 
@@ -829,12 +845,12 @@ class CopyFromTest(TestBase):
             batch_op.drop_index('ix_data')
 
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data VARCHAR(50), x INTEGER, PRIMARY KEY (id))',
-            'INSERT INTO _alembic_batch_temp (id, data, x) '
+            'INSERT INTO _alembic_tmp_foo (id, data, x) '
             'SELECT foo.id, foo.data, foo.x FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo'
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo'
         )
 
     def test_create_drop_index_wo_always(self):
@@ -868,12 +884,12 @@ class CopyFromTest(TestBase):
                 'ix_data', ['data'], unique=True)
 
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data INTEGER, x INTEGER, PRIMARY KEY (id))',
-            'INSERT INTO _alembic_batch_temp (id, data, x) SELECT foo.id, '
+            'INSERT INTO _alembic_tmp_foo (id, data, x) SELECT foo.id, '
             'CAST(foo.data AS INTEGER) AS anon_1, foo.x FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo',
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo',
             'CREATE UNIQUE INDEX ix_data ON foo (data)',
         )
 
@@ -886,17 +902,16 @@ class CopyFromTest(TestBase):
             batch_op.alter_column('data', type_=String)
 
         context.assert_(
-            'CREATE TABLE _alembic_batch_temp (id INTEGER NOT NULL, '
+            'CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, '
             'data VARCHAR, x INTEGER, PRIMARY KEY (id))',
-            'INSERT INTO _alembic_batch_temp (id, data, x) SELECT foo.id, '
-            'CAST(foo.data AS VARCHAR) AS anon_1, foo.x FROM foo',
+            'INSERT INTO _alembic_tmp_foo (id, data, x) SELECT foo.id, '
+            'foo.data, foo.x FROM foo',
             'DROP TABLE foo',
-            'ALTER TABLE _alembic_batch_temp RENAME TO foo'
+            'ALTER TABLE _alembic_tmp_foo RENAME TO foo'
         )
 
 
 class BatchRoundTripTest(TestBase):
-    __requires__ = ('sqlalchemy_08', )
     __only_on__ = "sqlite"
 
     def setUp(self):
@@ -970,6 +985,14 @@ class BatchRoundTripTest(TestBase):
         )
         t.create(self.conn)
 
+    def _timestamp_fixture(self):
+        t = Table(
+            'hasts', self.metadata,
+            Column('x', DateTime()),
+        )
+        t.create(self.conn)
+        return t
+
     def _int_to_boolean_fixture(self):
         t = Table(
             'hasbool', self.metadata,
@@ -993,6 +1016,23 @@ class BatchRoundTripTest(TestBase):
             [Integer]
         )
 
+    def test_no_net_change_timestamp(self):
+        t = self._timestamp_fixture()
+
+        import datetime
+        self.conn.execute(
+            t.insert(),
+            {"x": datetime.datetime(2012, 5, 18, 15, 32, 5)}
+        )
+
+        with self.op.batch_alter_table("hasts") as batch_op:
+            batch_op.alter_column("x", type_=DateTime())
+
+        eq_(
+            self.conn.execute(select([t.c.x])).fetchall(),
+            [(datetime.datetime(2012, 5, 18, 15, 32, 5),)]
+        )
+
     def test_drop_col_schematype(self):
         self._boolean_fixture()
         with self.op.batch_alter_table(
@@ -1060,7 +1100,7 @@ class BatchRoundTripTest(TestBase):
 
     # in particular, this tests that the failures
     # on PG and MySQL result in recovery of the batch system,
-    # e.g. that the _alembic_batch_temp table is dropped
+    # e.g. that the _alembic_tmp_temp table is dropped
     @config.requirements.no_referential_integrity
     def test_fk_points_to_me_recreate(self):
         self._test_fk_points_to_me("always")
@@ -1160,6 +1200,36 @@ class BatchRoundTripTest(TestBase):
             {"id": 5, "x": 9}
         ])
 
+    def test_drop_pk_col_readd_col(self):
+        # drop a column, add it back without primary_key=True, should no
+        # longer be in the constraint
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.drop_column('id')
+            batch_op.add_column(Column('id', Integer))
+
+        pk_const = Inspector.from_engine(self.conn).get_pk_constraint('foo')
+        eq_(pk_const['constrained_columns'], [])
+
+    def test_drop_pk_col_readd_pk_col(self):
+        # drop a column, add it back with primary_key=True, should remain
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.drop_column('id')
+            batch_op.add_column(Column('id', Integer, primary_key=True))
+
+        pk_const = Inspector.from_engine(self.conn).get_pk_constraint('foo')
+        eq_(pk_const['constrained_columns'], ['id'])
+
+    def test_drop_pk_col_readd_col_also_pk_const(self):
+        # drop a column, add it back without primary_key=True, but then
+        # also make anew PK constraint that includes it, should remain
+        with self.op.batch_alter_table("foo") as batch_op:
+            batch_op.drop_column('id')
+            batch_op.add_column(Column('id', Integer))
+            batch_op.create_primary_key('newpk', ['id'])
+
+        pk_const = Inspector.from_engine(self.conn).get_pk_constraint('foo')
+        eq_(pk_const['constrained_columns'], ['id'])
+
     def test_add_pk_constraint(self):
         self._no_pk_fixture()
         with self.op.batch_alter_table("nopk", recreate="always") as batch_op:
@@ -1268,7 +1338,11 @@ class BatchRoundTripTest(TestBase):
         bar.create(self.conn)
         self.conn.execute(bar.insert(), {'id': 1, 'flag': True})
         self.conn.execute(bar.insert(), {'id': 2, 'flag': False})
-        self.conn.execute(bar.insert(), {'id': 3, 'flag': 5})
+        self.conn.execute(
+            # override Boolean type which as of 1.1 coerces numerics
+            # to 1/0
+            text("insert into bar (id, flag) values (:id, :flag)"),
+            {'id': 3, 'flag': 5})
 
         with self.op.batch_alter_table(
             "bar",
@@ -1374,6 +1448,17 @@ class BatchRoundTripTest(TestBase):
 
 class BatchRoundTripMySQLTest(BatchRoundTripTest):
     __only_on__ = "mysql"
+    __backend__ = True
+
+    @exclusions.fails()
+    def test_drop_pk_col_readd_pk_col(self):
+        super(BatchRoundTripMySQLTest, self).test_drop_pk_col_readd_pk_col()
+
+    @exclusions.fails()
+    def test_drop_pk_col_readd_col_also_pk_const(self):
+        super(
+            BatchRoundTripMySQLTest, self
+        ).test_drop_pk_col_readd_col_also_pk_const()
 
     @exclusions.fails()
     def test_rename_column_pk(self):
@@ -1390,9 +1475,34 @@ class BatchRoundTripMySQLTest(BatchRoundTripTest):
     def test_create_drop_index(self):
         super(BatchRoundTripMySQLTest, self).test_create_drop_index()
 
+    # fails on mariadb 10.2, succeeds on 10.3
+    @exclusions.fails_if(config.requirements._mariadb_only_102)
+    def test_rename_column_boolean(self):
+        super(BatchRoundTripMySQLTest, self).test_rename_column_boolean()
+
+    @config.requirements.mysql_check_reflection_or_none
+    def test_change_type_boolean_to_int(self):
+        super(BatchRoundTripMySQLTest, self).test_change_type_boolean_to_int()
+
+    @config.requirements.mysql_check_reflection_or_none
+    def test_change_type_int_to_boolean(self):
+        super(BatchRoundTripMySQLTest, self).test_change_type_int_to_boolean()
+
 
 class BatchRoundTripPostgresqlTest(BatchRoundTripTest):
     __only_on__ = "postgresql"
+    __backend__ = True
+
+    @exclusions.fails()
+    def test_drop_pk_col_readd_pk_col(self):
+        super(
+            BatchRoundTripPostgresqlTest, self).test_drop_pk_col_readd_pk_col()
+
+    @exclusions.fails()
+    def test_drop_pk_col_readd_col_also_pk_const(self):
+        super(
+            BatchRoundTripPostgresqlTest, self
+        ).test_drop_pk_col_readd_col_also_pk_const()
 
     @exclusions.fails()
     def test_change_type(self):
