@@ -1,6 +1,7 @@
 from alembic import command
 from io import TextIOWrapper, BytesIO
 from alembic.script import ScriptDirectory
+from alembic import config
 from alembic.testing.fixtures import TestBase, capture_context_buffer
 from alembic.testing.env import staging_env, _sqlite_testing_config, \
     three_rev_fixture, clear_staging_env, _no_sql_testing_config, \
@@ -29,22 +30,61 @@ class HistoryTest(_BufMixin, TestBase):
         cls.env = staging_env()
         cls.cfg = _sqlite_testing_config()
         cls.a, cls.b, cls.c = three_rev_fixture(cls.cfg)
+        cls._setup_env_file()
 
     @classmethod
     def teardown_class(cls):
         clear_staging_env()
 
-    def _eq_cmd_output(self, buf, expected):
+    def teardown(self):
+        self.cfg.set_main_option("revision_environment", "false")
+
+    @classmethod
+    def _setup_env_file(self):
+        env_file_fixture(r"""
+
+from sqlalchemy import MetaData, engine_from_config
+target_metadata = MetaData()
+
+engine = engine_from_config(
+    config.get_section(config.config_ini_section),
+    prefix='sqlalchemy.')
+
+connection = engine.connect()
+
+context.configure(
+    connection=connection, target_metadata=target_metadata
+)
+
+try:
+    with context.begin_transaction():
+        config.stdout.write(u"environment included OK\n")
+        context.run_migrations()
+finally:
+    connection.close()
+
+""")
+
+    def _eq_cmd_output(self, buf, expected, env_token=False, currents=()):
         script = ScriptDirectory.from_config(self.cfg)
 
         # test default encode/decode behavior as well,
         # rev B has a non-ascii char in it + a coding header.
+
+        assert_lines = []
+        for _id in expected:
+            rev = script.get_revision(_id)
+            if _id in currents:
+                rev._db_current_indicator = True
+            assert_lines.append(rev.log_entry)
+
+        if env_token:
+            assert_lines.insert(0, "environment included OK")
+
         eq_(
             buf.getvalue().decode("ascii", 'replace').strip(),
-            "\n".join([
-                script.get_revision(rev).log_entry
-                for rev in expected
-            ]).encode("ascii", "replace").decode("ascii").strip()
+            "\n".join(assert_lines).
+            encode("ascii", "replace").decode("ascii").strip()
         )
 
     def test_history_full(self):
@@ -52,11 +92,23 @@ class HistoryTest(_BufMixin, TestBase):
         command.history(self.cfg, verbose=True)
         self._eq_cmd_output(buf, [self.c, self.b, self.a])
 
+    def test_history_full_environment(self):
+        self.cfg.stdout = buf = self._buf_fixture()
+        self.cfg.set_main_option("revision_environment", "true")
+        command.history(self.cfg, verbose=True)
+        self._eq_cmd_output(buf, [self.c, self.b, self.a], env_token=True)
+
     def test_history_num_range(self):
         self.cfg.stdout = buf = self._buf_fixture()
         command.history(self.cfg, "%s:%s" % (self.a, self.b), verbose=True)
         self._eq_cmd_output(buf, [self.b, self.a])
 
+    def test_history_num_range_environment(self):
+        self.cfg.stdout = buf = self._buf_fixture()
+        self.cfg.set_main_option("revision_environment", "true")
+        command.history(self.cfg, "%s:%s" % (self.a, self.b), verbose=True)
+        self._eq_cmd_output(buf, [self.b, self.a], env_token=True)
+
     def test_history_base_to_num(self):
         self.cfg.stdout = buf = self._buf_fixture()
         command.history(self.cfg, ":%s" % (self.b), verbose=True)
@@ -67,6 +119,12 @@ class HistoryTest(_BufMixin, TestBase):
         command.history(self.cfg, "%s:" % (self.a), verbose=True)
         self._eq_cmd_output(buf, [self.c, self.b, self.a])
 
+    def test_history_num_to_head_environment(self):
+        self.cfg.stdout = buf = self._buf_fixture()
+        self.cfg.set_main_option("revision_environment", "true")
+        command.history(self.cfg, "%s:" % (self.a), verbose=True)
+        self._eq_cmd_output(buf, [self.c, self.b, self.a], env_token=True)
+
     def test_history_num_plus_relative(self):
         self.cfg.stdout = buf = self._buf_fixture()
         command.history(self.cfg, "%s:+2" % (self.a), verbose=True)
@@ -86,13 +144,26 @@ class HistoryTest(_BufMixin, TestBase):
         command.stamp(self.cfg, self.b)
         self.cfg.stdout = buf = self._buf_fixture()
         command.history(self.cfg, "current:", verbose=True)
-        self._eq_cmd_output(buf, [self.c, self.b])
+        self._eq_cmd_output(buf, [self.c, self.b], env_token=True)
 
     def test_history_current_to_head_as_base(self):
         command.stamp(self.cfg, "base")
         self.cfg.stdout = buf = self._buf_fixture()
         command.history(self.cfg, "current:", verbose=True)
-        self._eq_cmd_output(buf, [self.c, self.b, self.a])
+        self._eq_cmd_output(buf, [self.c, self.b, self.a], env_token=True)
+
+    def test_history_include_env(self):
+        self.cfg.stdout = buf = self._buf_fixture()
+        self.cfg.set_main_option("revision_environment", "true")
+        command.history(self.cfg, verbose=True)
+        self._eq_cmd_output(buf, [self.c, self.b, self.a], env_token=True)
+
+    def test_history_indicate_current(self):
+        command.stamp(self.cfg, (self.b,))
+        self.cfg.stdout = buf = self._buf_fixture()
+        command.history(self.cfg, indicate_current=True, verbose=True)
+        self._eq_cmd_output(
+            buf, [self.c, self.b, self.a], currents=(self.b,), env_token=True)
 
 
 class CurrentTest(_BufMixin, TestBase):
@@ -603,3 +674,45 @@ class EditTest(TestBase):
         with mock.patch('alembic.util.edit') as edit:
             command.edit(self.cfg, "current")
             edit.assert_called_with(expected_call_arg)
+
+
+class CommandLineTest(TestBase):
+    @classmethod
+    def setup_class(cls):
+        cls.env = staging_env()
+        cls.cfg = _sqlite_testing_config()
+        cls.a, cls.b, cls.c = three_rev_fixture(cls.cfg)
+
+    def test_run_cmd_args_missing(self):
+        canary = mock.Mock()
+
+        orig_revision = command.revision
+
+        # the command function has "process_revision_directives"
+        # however the ArgumentParser does not.  ensure things work
+        def revision(
+            config, message=None, autogenerate=False, sql=False,
+            head="head", splice=False, branch_label=None,
+            version_path=None, rev_id=None, depends_on=None,
+            process_revision_directives=None
+        ):
+            canary(
+                config, message=message
+            )
+
+        revision.__module__ = 'alembic.command'
+
+        # CommandLine() pulls the function into the ArgumentParser
+        # and needs the full signature, so we can't patch the "revision"
+        # command normally as ArgumentParser gives us no way to get to it.
+        config.command.revision = revision
+        try:
+            commandline = config.CommandLine()
+            options = commandline.parser.parse_args(["revision", "-m", "foo"])
+            commandline.run_cmd(self.cfg, options)
+        finally:
+            config.command.revision = orig_revision
+        eq_(
+            canary.mock_calls,
+            [mock.call(self.cfg, message="foo")]
+        )
