@@ -1,9 +1,8 @@
 #!coding: utf-8
 
-import os
-import tempfile
 
-from alembic import config, util, compat
+from alembic import config, util
+from alembic.util import compat
 from alembic.migration import MigrationContext
 from alembic.operations import Operations
 from alembic.script import ScriptDirectory
@@ -17,6 +16,7 @@ from alembic.testing.env import _no_sql_testing_config, clear_staging_env,\
 
 
 class FileConfigTest(TestBase):
+
     def test_config_args(self):
         cfg = _write_config_file("""
 [alembic]
@@ -50,6 +50,29 @@ class ConfigTest(TestBase):
         cfg.set_section_option("foo", "echo", "True")
         eq_(cfg.get_section_option("foo", "echo"), "True")
 
+    def test_config_set_main_option_percent(self):
+        cfg = config.Config()
+        cfg.set_main_option("foob", "a %% percent")
+
+        eq_(cfg.get_main_option("foob"), "a % percent")
+
+    def test_config_set_section_option_percent(self):
+        cfg = config.Config()
+        cfg.set_section_option("some_section", "foob", "a %% percent")
+
+        eq_(cfg.get_section_option("some_section", "foob"), "a % percent")
+
+    def test_config_set_section_option_interpolation(self):
+        cfg = config.Config()
+        cfg.set_section_option("some_section", "foob", "foob_value")
+
+        cfg.set_section_option(
+            "some_section", "bar", "bar with %(foob)s")
+
+        eq_(
+            cfg.get_section_option("some_section", "bar"),
+            "bar with foob_value")
+
     def test_standalone_op(self):
         eng, buf = capture_db()
 
@@ -114,6 +137,15 @@ class StdoutOutputEncodingTest(TestBase):
             [call.write('m?il x y'), call.write('\n')]
         )
 
+    def test_only_formats_output_with_args(self):
+        stdout = Mock(encoding=None)
+        cfg = config.Config(stdout=stdout)
+        cfg.print_stdout(compat.u("test 3%"))
+        eq_(
+            stdout.mock_calls,
+            [call.write('test 3%'), call.write('\n')]
+        )
+
 
 class TemplateOutputEncodingTest(TestBase):
     def setUp(self):
