@@ -3,6 +3,7 @@ from sqlalchemy import DateTime, MetaData, Table, Column, text, Integer, \
     String, Interval, Sequence, Numeric, BigInteger, Float, Numeric
 from sqlalchemy.dialects.postgresql import ARRAY, UUID, BYTEA
 from sqlalchemy.engine.reflection import Inspector
+from sqlalchemy import types
 from alembic.operations import Operations
 from sqlalchemy.sql import table, column
 from alembic.autogenerate.compare import \
@@ -31,7 +32,7 @@ from sqlalchemy.sql import false
 
 
 if util.sqla_09:
-    from sqlalchemy.dialects.postgresql import JSON, JSONB
+    from sqlalchemy.dialects.postgresql import JSON, JSONB, HSTORE
 
 
 class PostgresqlOpTest(TestBase):
@@ -46,7 +47,6 @@ class PostgresqlOpTest(TestBase):
         op.rename_table('t1', 't2', schema="foo")
         context.assert_("ALTER TABLE foo.t1 RENAME TO t2")
 
-    @config.requirements.fail_before_sqla_080
     def test_create_index_postgresql_expressions(self):
         context = op_fixture("postgresql")
         op.create_index(
@@ -69,6 +69,27 @@ class PostgresqlOpTest(TestBase):
             "CREATE INDEX geocoded ON locations (coordinates) "
             "WHERE locations.coordinates != Null")
 
+    @config.requirements.fail_before_sqla_099
+    def test_create_index_postgresql_concurrently(self):
+        context = op_fixture("postgresql")
+        op.create_index(
+            'geocoded',
+            'locations',
+            ['coordinates'],
+            postgresql_concurrently=True)
+        context.assert_(
+            "CREATE INDEX CONCURRENTLY geocoded ON locations (coordinates)")
+
+    @config.requirements.fail_before_sqla_110
+    def test_drop_index_postgresql_concurrently(self):
+        context = op_fixture("postgresql")
+        op.drop_index(
+            'geocoded',
+            'locations',
+            postgresql_concurrently=True)
+        context.assert_(
+            "DROP INDEX CONCURRENTLY geocoded")
+
     def test_alter_column_type_using(self):
         context = op_fixture('postgresql')
         op.alter_column("t", "c", type_=Integer, postgresql_using='c::integer')
@@ -104,7 +125,7 @@ class PostgresqlOpTest(TestBase):
             '("SomeColumn" WITH >) WHERE ("SomeColumn" > 5)'
         )
 
-    @config.requirements.fail_before_sqla_100
+    @config.requirements.fail_before_sqla_1010
     def test_create_exclude_constraint_quoted_column(self):
         context = op_fixture("postgresql")
         op.create_exclude_constraint(
@@ -174,7 +195,6 @@ def downgrade():
 
 """ % self.rid)
 
-    @config.requirements.sqlalchemy_09
     def test_offline_inline_enum_create(self):
         self._inline_enum_script()
         with capture_context_buffer() as buf:
@@ -191,7 +211,6 @@ def downgrade():
         # no drop since we didn't emit events
         assert "DROP TYPE pgenum" not in buf.getvalue()
 
-    @config.requirements.sqlalchemy_09
     def test_offline_distinct_enum_create(self):
         self._distinct_enum_script()
         with capture_context_buffer() as buf:
@@ -210,6 +229,7 @@ def downgrade():
 
 class PostgresqlInlineLiteralTest(TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
@@ -256,6 +276,8 @@ class PostgresqlInlineLiteralTest(TestBase):
 
 class PostgresqlDefaultCompareTest(TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
+
 
     @classmethod
     def setup_class(cls):
@@ -491,6 +513,7 @@ class PostgresqlDefaultCompareTest(TestBase):
 
 class PostgresqlDetectSerialTest(TestBase):
     __only_on__ = 'postgresql'
+    __backend__ = True
 
     @classmethod
     def setup_class(cls):
@@ -612,20 +635,12 @@ class PostgresqlAutogenRenderTest(TestBase):
 
         op_obj = ops.CreateIndexOp.from_index(idx)
 
-        if util.sqla_08:
-            eq_ignore_whitespace(
-                autogenerate.render_op_text(autogen_context, op_obj),
-                """op.create_index('foo_idx', 't', \
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(autogen_context, op_obj),
+            """op.create_index('foo_idx', 't', \
 ['x', 'y'], unique=False, """
-                """postgresql_where=sa.text(!U"y = 'something'"))"""
-            )
-        else:
-            eq_ignore_whitespace(
-                autogenerate.render_op_text(autogen_context, op_obj),
-                """op.create_index('foo_idx', 't', ['x', 'y'], \
-unique=False, """
-                """postgresql_where=sa.text(!U't.y = %(y_1)s'))"""
-            )
+            """postgresql_where=sa.text(!U"y = 'something'"))"""
+        )
 
     def test_render_server_default_native_boolean(self):
         c = Column(
@@ -642,8 +657,7 @@ unique=False, """
             'nullable=False)'
         )
 
-    @config.requirements.sqlalchemy_09
-    def test_array_type(self):
+    def test_postgresql_array_type(self):
 
         eq_ignore_whitespace(
             autogenerate.render._repr_type(
@@ -667,7 +681,57 @@ unique=False, """
         assert 'from sqlalchemy.dialects import postgresql' in \
             self.autogen_context.imports
 
-    @config.requirements.sqlalchemy_09
+    @config.requirements.sqlalchemy_110
+    def test_postgresql_hstore_subtypes(self):
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                HSTORE(), self.autogen_context),
+            "postgresql.HSTORE(text_type=sa.Text())"
+        )
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                HSTORE(text_type=String()), self.autogen_context),
+            "postgresql.HSTORE(text_type=sa.String())"
+        )
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                HSTORE(text_type=BYTEA()), self.autogen_context),
+            "postgresql.HSTORE(text_type=postgresql.BYTEA())"
+        )
+
+        assert 'from sqlalchemy.dialects import postgresql' in \
+            self.autogen_context.imports
+
+    @config.requirements.sqlalchemy_110
+    def test_generic_array_type(self):
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                types.ARRAY(Integer), self.autogen_context),
+            "sa.ARRAY(sa.Integer())"
+        )
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                types.ARRAY(DateTime(timezone=True)), self.autogen_context),
+            "sa.ARRAY(sa.DateTime(timezone=True))"
+        )
+
+        assert 'from sqlalchemy.dialects import postgresql' not in \
+            self.autogen_context.imports
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                types.ARRAY(BYTEA, as_tuple=True, dimensions=2),
+                self.autogen_context),
+            "sa.ARRAY(postgresql.BYTEA(), as_tuple=True, dimensions=2)"
+        )
+
+        assert 'from sqlalchemy.dialects import postgresql' in \
+            self.autogen_context.imports
+
     def test_array_type_user_defined_inner(self):
         def repr_type(typestring, object_, autogen_context):
             if typestring == 'type' and isinstance(object_, String):
@@ -706,10 +770,36 @@ unique=False, """
 
         eq_ignore_whitespace(
             autogenerate.render_op_text(autogen_context, op_obj),
-            "op.create_exclude_constraint('t_excl_x', 't', ('x', '>'), "
+            "op.create_exclude_constraint('t_excl_x', 't', (sa.column('x'), '>'), "
             "where=sa.text(!U'x != 2'), using='gist')"
         )
 
+    @config.requirements.fail_before_sqla_100
+    def test_add_exclude_constraint_case_sensitive(self):
+        from sqlalchemy.dialects.postgresql import ExcludeConstraint
+
+        autogen_context = self.autogen_context
+
+        m = MetaData()
+        t = Table('TTAble', m,
+                  Column('XColumn', String),
+                  Column('YColumn', String)
+                  )
+
+        op_obj = ops.AddConstraintOp.from_constraint(ExcludeConstraint(
+            (t.c.XColumn, ">"),
+            where=t.c.XColumn != 2,
+            using="gist",
+            name="t_excl_x"
+        ))
+
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(autogen_context, op_obj),
+            "op.create_exclude_constraint('t_excl_x', 'TTAble', (sa.column('XColumn'), '>'), "
+            "where=sa.text(!U'\"XColumn\" != 2'), using='gist')"
+        )
+
+
     @config.requirements.fail_before_sqla_100
     def test_inline_exclude_constraint(self):
         from sqlalchemy.dialects.postgresql import ExcludeConstraint
@@ -740,7 +830,37 @@ unique=False, """
             ")"
         )
 
-    @config.requirements.sqlalchemy_09
+    @config.requirements.fail_before_sqla_100
+    def test_inline_exclude_constraint_case_sensitive(self):
+        from sqlalchemy.dialects.postgresql import ExcludeConstraint
+
+        autogen_context = self.autogen_context
+
+        m = MetaData()
+        t = Table(
+            'TTable', m,
+            Column('XColumn', String),
+            Column('YColumn', String),
+        )
+        ExcludeConstraint(
+            (t.c.XColumn, ">"),
+            using="gist",
+            where='"XColumn" != 2',
+            name="TExclX"
+        )
+
+        op_obj = ops.CreateTableOp.from_table(t)
+
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(autogen_context, op_obj),
+            "op.create_table('TTable',sa.Column('XColumn', sa.String(), "
+            "nullable=True),"
+            "sa.Column('YColumn', sa.String(), nullable=True),"
+            "postgresql.ExcludeConstraint((sa.column('XColumn'), '>'), "
+            "where=sa.text(!U'\"XColumn\" != 2'), using='gist', "
+            "name='TExclX'))"
+        )
+
     def test_json_type(self):
         if config.requirements.sqlalchemy_110.enabled:
             eq_ignore_whitespace(
@@ -755,7 +875,6 @@ unique=False, """
                 "postgresql.JSON()"
             )
 
-    @config.requirements.sqlalchemy_09
     def test_jsonb_type(self):
         if config.requirements.sqlalchemy_110.enabled:
             eq_ignore_whitespace(
