@@ -2,19 +2,21 @@
 
 import os
 import re
+import textwrap
 
 from alembic import command, util
 from alembic.util import compat
 from alembic.script import ScriptDirectory, Script
 from alembic.testing.env import clear_staging_env, staging_env, \
     _sqlite_testing_config, write_script, _sqlite_file_db, \
-    three_rev_fixture, _no_sql_testing_config
+    three_rev_fixture, _no_sql_testing_config, env_file_fixture
 from alembic.testing import eq_, assert_raises_message
 from alembic.testing.fixtures import TestBase, capture_context_buffer
 from alembic.environment import EnvironmentContext
 from contextlib import contextmanager
 from alembic.testing import mock
 
+
 class ApplyVersionsFunctionalTest(TestBase):
     __only_on__ = 'sqlite'
 
@@ -35,6 +37,7 @@ class ApplyVersionsFunctionalTest(TestBase):
         self._test_004_downgrade()
         self._test_005_upgrade()
         self._test_006_upgrade_again()
+        self._test_007_stamp_upgrade()
 
     def _test_001_revisions(self):
         self.a = a = util.rev_id()
@@ -128,9 +131,111 @@ class ApplyVersionsFunctionalTest(TestBase):
         assert db.dialect.has_table(db.connect(), 'bar')
         assert not db.dialect.has_table(db.connect(), 'bat')
 
+    def _test_007_stamp_upgrade(self):
+        command.stamp(self.cfg, self.c)
+        db = self.bind
+        assert db.dialect.has_table(db.connect(), 'foo')
+        assert db.dialect.has_table(db.connect(), 'bar')
+        assert not db.dialect.has_table(db.connect(), 'bat')
 
-class SourcelessApplyVersionsTest(ApplyVersionsFunctionalTest):
-    sourceless = True
+
+class SimpleSourcelessApplyVersionsTest(ApplyVersionsFunctionalTest):
+    sourceless = "simple"
+
+
+class NewFangledSourcelessEnvOnlyApplyVersionsTest(
+        ApplyVersionsFunctionalTest):
+    sourceless = "pep3147_envonly"
+
+    __requires__ = "pep3147",
+
+
+class NewFangledSourcelessEverythingApplyVersionsTest(
+        ApplyVersionsFunctionalTest):
+    sourceless = "pep3147_everything"
+
+    __requires__ = "pep3147",
+
+
+class CallbackEnvironmentTest(ApplyVersionsFunctionalTest):
+    exp_kwargs = frozenset(('ctx', 'heads', 'run_args', 'step'))
+
+    @staticmethod
+    def _env_file_fixture():
+        env_file_fixture(textwrap.dedent("""\
+            import alembic
+            from alembic import context
+            from sqlalchemy import engine_from_config, pool
+
+            config = context.config
+
+            target_metadata = None
+
+            def run_migrations_offline():
+                url = config.get_main_option('sqlalchemy.url')
+                context.configure(
+                    url=url, target_metadata=target_metadata,
+                    on_version_apply=alembic.mock_event_listener,
+                    literal_binds=True)
+
+                with context.begin_transaction():
+                    context.run_migrations()
+
+            def run_migrations_online():
+                connectable = engine_from_config(
+                    config.get_section(config.config_ini_section),
+                    prefix='sqlalchemy.',
+                    poolclass=pool.NullPool)
+                with connectable.connect() as connection:
+                    context.configure(
+                        connection=connection,
+                        on_version_apply=alembic.mock_event_listener,
+                        target_metadata=target_metadata,
+                    )
+                    with context.begin_transaction():
+                        context.run_migrations()
+
+            if context.is_offline_mode():
+                run_migrations_offline()
+            else:
+                run_migrations_online()
+            """))
+
+    def test_steps(self):
+        import alembic
+        alembic.mock_event_listener = None
+        self._env_file_fixture()
+        with mock.patch('alembic.mock_event_listener', mock.Mock()) as mymock:
+            super(CallbackEnvironmentTest, self).test_steps()
+        calls = mymock.call_args_list
+        assert calls
+        for call in calls:
+            args, kw = call
+            assert not args
+            assert set(kw.keys()) >= self.exp_kwargs
+            assert kw['run_args'] == {}
+            assert hasattr(kw['ctx'], 'get_current_revision')
+
+            step = kw['step']
+            assert isinstance(step.is_upgrade, bool)
+            assert isinstance(step.is_stamp, bool)
+            assert isinstance(step.is_migration, bool)
+            assert isinstance(step.up_revision_id, compat.string_types)
+            assert isinstance(step.up_revision, Script)
+
+            for revtype in 'up', 'down', 'source', 'destination':
+                revs = getattr(step, '%s_revisions' % revtype)
+                assert isinstance(revs, tuple)
+                for rev in revs:
+                    assert isinstance(rev, Script)
+                revids = getattr(step, '%s_revision_ids' % revtype)
+                for revid in revids:
+                    assert isinstance(revid, compat.string_types)
+
+            heads = kw['heads']
+            assert hasattr(heads, '__iter__')
+            for h in heads:
+                assert h is None or isinstance(h, compat.string_types)
 
 
 class OfflineTransactionalDDLTest(TestBase):
@@ -281,6 +386,35 @@ def downgrade():
                 transactional_ddl=True, transaction_per_migration=False):
             command.upgrade(self.cfg, c)
 
+    def test_noerr_transaction_opened_externally(self):
+        a, b, c = self._opened_transaction_fixture()
+
+        env_file_fixture("""
+from sqlalchemy import engine_from_config, pool
+
+def run_migrations_online():
+    connectable = engine_from_config(
+        config.get_section(config.config_ini_section),
+        prefix='sqlalchemy.',
+        poolclass=pool.NullPool)
+
+    with connectable.connect() as connection:
+        with connection.begin() as real_trans:
+            context.configure(
+                connection=connection,
+                transactional_ddl=False,
+                transaction_per_migration=False
+            )
+
+            with context.begin_transaction():
+                context.run_migrations()
+
+run_migrations_online()
+
+""")
+
+        command.stamp(self.cfg, c)
+
 
 class EncodingTest(TestBase):
 
@@ -402,7 +536,7 @@ def downgrade():
 
 """)
         pyc_path = util.pyc_file_from_path(path)
-        if os.access(pyc_path, os.F_OK):
+        if pyc_path is not None and os.access(pyc_path, os.F_OK):
             os.unlink(pyc_path)
 
         assert_raises_message(
@@ -472,8 +606,20 @@ class IgnoreFilesTest(TestBase):
         self._test_ignore_dot_hash_py("pyo")
 
 
-class SourcelessIgnoreFilesTest(IgnoreFilesTest):
-    sourceless = True
+class SimpleSourcelessIgnoreFilesTest(IgnoreFilesTest):
+    sourceless = "simple"
+
+
+class NewFangledEnvOnlySourcelessIgnoreFilesTest(IgnoreFilesTest):
+    sourceless = "pep3147_envonly"
+
+    __requires__ = "pep3147",
+
+
+class NewFangledEverythingSourcelessIgnoreFilesTest(IgnoreFilesTest):
+    sourceless = "pep3147_everything"
+
+    __requires__ = "pep3147",
 
 
 class SourcelessNeedsFlagTest(TestBase):
