@@ -18,6 +18,7 @@ import sqlalchemy as sa
 from sqlalchemy.engine.reflection import Inspector
 from alembic.util import CommandError
 import re
+from dateutil import tz
 
 env, abc, def_ = None, None, None
 
@@ -153,6 +154,80 @@ class ScriptNamingTest(TestBase):
                 "message_2012_7_25_15_8_5.py" % _get_staging_directory())
         )
 
+    def _test_tz(self, timezone_arg, given, expected):
+        script = ScriptDirectory(
+            _get_staging_directory(),
+            file_template="%(rev)s_%(slug)s_"
+            "%(year)s_%(month)s_"
+            "%(day)s_%(hour)s_"
+            "%(minute)s_%(second)s",
+            timezone=timezone_arg
+        )
+
+        with mock.patch(
+                "alembic.script.base.datetime",
+                mock.Mock(
+                    datetime=mock.Mock(
+                        utcnow=lambda: given,
+                        now=lambda: given
+                    )
+                )
+        ):
+            create_date = script._generate_create_date()
+        eq_(
+            create_date,
+            expected
+        )
+
+    def test_custom_tz(self):
+        self._test_tz(
+            'EST5EDT',
+            datetime.datetime(2012, 7, 25, 15, 8, 5),
+            datetime.datetime(
+                2012, 7, 25, 11, 8, 5, tzinfo=tz.gettz('EST5EDT'))
+        )
+
+    def test_custom_tz_lowercase(self):
+        self._test_tz(
+            'est5edt',
+            datetime.datetime(2012, 7, 25, 15, 8, 5),
+            datetime.datetime(
+                2012, 7, 25, 11, 8, 5, tzinfo=tz.gettz('EST5EDT'))
+        )
+
+    def test_custom_tz_utc(self):
+        self._test_tz(
+            'utc',
+            datetime.datetime(2012, 7, 25, 15, 8, 5),
+            datetime.datetime(
+                2012, 7, 25, 15, 8, 5, tzinfo=tz.gettz('UTC'))
+        )
+
+    def test_custom_tzdata_tz(self):
+        self._test_tz(
+            'Europe/Berlin',
+            datetime.datetime(2012, 7, 25, 15, 8, 5),
+            datetime.datetime(
+                2012, 7, 25, 17, 8, 5, tzinfo=tz.gettz('Europe/Berlin'))
+        )
+
+    def test_default_tz(self):
+        self._test_tz(
+            None,
+            datetime.datetime(2012, 7, 25, 15, 8, 5),
+            datetime.datetime(2012, 7, 25, 15, 8, 5)
+        )
+
+    def test_tz_cant_locate(self):
+        assert_raises_message(
+            CommandError,
+            "Can't locate timezone: fake",
+            self._test_tz,
+            "fake",
+            datetime.datetime(2012, 7, 25, 15, 8, 5),
+            datetime.datetime(2012, 7, 25, 15, 8, 5)
+        )
+
 
 class RevisionCommandTest(TestBase):
     def setUp(self):
@@ -188,6 +263,43 @@ class RevisionCommandTest(TestBase):
             self.cfg, message="some message", head=self.b
         )
 
+    def test_illegal_revision_chars(self):
+        assert_raises_message(
+            util.CommandError,
+            r"Character\(s\) '-' not allowed in "
+            "revision identifier 'no-dashes'",
+            command.revision,
+            self.cfg, message="some message", rev_id="no-dashes"
+        )
+
+        assert not os.path.exists(
+            os.path.join(
+                self.env.dir, "versions", "no-dashes_some_message.py"))
+
+        assert_raises_message(
+            util.CommandError,
+            r"Character\(s\) '@' not allowed in "
+            "revision identifier 'no@atsigns'",
+            command.revision,
+            self.cfg, message="some message", rev_id="no@atsigns"
+        )
+
+        assert_raises_message(
+            util.CommandError,
+            r"Character\(s\) '-, @' not allowed in revision "
+            "identifier 'no@atsigns-ordashes'",
+            command.revision,
+            self.cfg, message="some message", rev_id="no@atsigns-ordashes"
+        )
+
+        assert_raises_message(
+            util.CommandError,
+            r"Character\(s\) '\+' not allowed in revision "
+            r"identifier 'no\+plussignseither'",
+            command.revision,
+            self.cfg, message="some message", rev_id="no+plussignseither"
+        )
+
     def test_create_script_branches(self):
         rev = command.revision(
             self.cfg, message="some message", branch_label="foobar")
@@ -437,7 +549,8 @@ def downgrade():
                 head="model1@head",
                 process_revision_directives=process_revision_directives)
 
-        result = open(rev.path).read()
+        with open(rev.path) as handle:
+            result = handle.read()
         assert ("""
 def upgrade():
     # ### commands auto generated by Alembic - please adjust! ###
@@ -492,7 +605,7 @@ class ImportsTest(TestBase):
     def tearDown(self):
         clear_staging_env()
 
-    def _env_fixture(self, target_metadata):
+    def _env_fixture(self, target_metadata, **kw):
         self.engine = engine = _sqlite_file_db()
 
         def run_env(self):
@@ -501,7 +614,8 @@ class ImportsTest(TestBase):
             with engine.connect() as connection:
                 context.configure(
                     connection=connection,
-                    target_metadata=target_metadata)
+                    target_metadata=target_metadata,
+                    **kw)
                 with context.begin_transaction():
                     context.run_migrations()
 
@@ -523,13 +637,22 @@ class ImportsTest(TestBase):
             Column('x', type_)
         )
 
-        with self._env_fixture(m):
+        def process_revision_directives(context, rev, generate_revisions):
+            generate_revisions[0].imports.add(
+                "from sqlalchemy.dialects.mysql import TINYINT")
+
+        with self._env_fixture(
+                m,
+                process_revision_directives=process_revision_directives
+        ):
             rev = command.revision(
                 self.cfg, message="some message",
                 autogenerate=True)
 
         with open(rev.path) as file_:
-            assert "from sqlalchemy.dialects import mysql" in file_.read()
+            contents = file_.read()
+            assert "from sqlalchemy.dialects import mysql" in contents
+            assert "from sqlalchemy.dialects.mysql import TINYINT" in contents
 
 
 class MultiContextTest(TestBase):
@@ -873,7 +996,8 @@ context.configure(dialect_name='sqlite', template_args={"somearg":"somevalue"})
                 str(ce)
             )
             assert m, "Command error did not produce a file"
-            contents = open(m.group(1)).read()
+            with open(m.group(1)) as handle:
+                contents = handle.read()
             os.remove(m.group(1))
             assert "<% z = x + y %>" in contents
 
